@@ -42,6 +42,7 @@
 #include <type_traits>
 #include <utility>
 
+#include "absl/base/nullability.h"
 #include "base/win32/hresult.h"
 #include "base/win32/hresultor.h"
 
@@ -54,7 +55,7 @@ concept ComInterface = std::derived_from<I, IUnknown> && std::is_abstract_v<I>;
 // MakeComPtr is like std::make_unique but for COM pointers. Returns nullptr if
 // new fails.
 template <typename T, typename... Args>
-wil::com_ptr_nothrow<T> MakeComPtr(Args &&...args) {
+wil::com_ptr_nothrow<T> MakeComPtr(Args&&... args) {
   static_assert(std::is_base_of_v<IUnknown, T>, "T must implement IUnknown.");
 
   return wil::com_ptr_nothrow<T>(new (std::nothrow)
@@ -78,12 +79,12 @@ wil::com_ptr_nothrow<Interface> ComCreateInstance() {
 
 // Returns the result of QueryInterface as HResultOr<wil::com_ptr_nothrow<T>>.
 template <typename T, typename U>
-HResultOr<wil::com_ptr_nothrow<T>> ComQueryHR(U &&source) {
+HResultOr<wil::com_ptr_nothrow<T>> ComQueryHR(U&& source) {
   static_assert(ComInterface<T>, "T must be a COM interface.");
 
   wil::com_ptr_nothrow<T> result;
   // Workaround as WIL doesn't detect convertible queries with VC++2017.
-  auto *ptr = wil::com_raw_ptr(std::forward<U>(source));
+  auto* ptr = wil::com_raw_ptr(std::forward<U>(source));
   if constexpr (std::derived_from<std::remove_pointer_t<decltype(ptr)>, T>) {
     // The constructor of wil::com_ptr calls AddRef().
     return ptr;
@@ -99,11 +100,11 @@ HResultOr<wil::com_ptr_nothrow<T>> ComQueryHR(U &&source) {
 // Returns the result of QueryInterface as wil::com_ptr_nothrow<T>.
 // Use this function instead of wil::try_com_query_nothrow.
 template <typename T, typename U>
-wil::com_ptr_nothrow<T> ComQuery(U &&source) {
+wil::com_ptr_nothrow<T> ComQuery(U&& source) {
   static_assert(ComInterface<T>, "T must be a COM interface.");
 
   // Workaround as WIL doesn't detect convertible queries with VC++2017.
-  auto *ptr = wil::com_raw_ptr(std::forward<U>(source));
+  auto* ptr = wil::com_raw_ptr(std::forward<U>(source));
   if constexpr (std::derived_from<std::remove_pointer_t<decltype(ptr)>, T>) {
     // The constructor of wil::com_ptr calls AddRef().
     return ptr;
@@ -115,7 +116,7 @@ wil::com_ptr_nothrow<T> ComQuery(U &&source) {
 // Similar to ComQuery but returns nullptr if source is nullptr.
 // Use this function instead of wil::try_com_copy_nothrow.
 template <typename T, typename U>
-wil::com_ptr_nothrow<T> ComCopy(U &&source) {
+wil::com_ptr_nothrow<T> ComCopy(U&& source) {
   if (source) {
     return ComQuery<T>(std::forward<U>(source));
   } else {
@@ -130,10 +131,73 @@ inline wil::unique_bstr MakeUniqueBSTR(const std::wstring_view source) {
   return wil::unique_bstr(SysAllocStringLen(source.data(), source.size()));
 }
 
-inline wil::unique_bstr MakeUniqueBSTR(const wchar_t *source) {
+inline wil::unique_bstr MakeUniqueBSTR(const wchar_t* source) {
   return wil::unique_bstr(SysAllocString(source));
 }
 
+// Saves the value to the out parameter of a COM method.
+//
+// Return values:
+//   - S_OK: success.
+//   - E_INVALIDARG: the out parameter is null.
+//   - E_OUTOFMEMORY: the value is a null pointer.
+//
+// Note: this behavior is different from IUnknown methods. Check COM interface
+// documentation to make sure the returned error codes are correct.
+template <typename T, typename U>
+HResult SaveToOutParam(T value, U* absl_nullable out);
+template <typename T, typename U>
+HResult SaveToOutParam(T* absl_nullable value, U** absl_nullable out);
+template <typename T, typename U>
+HResult SaveToOutParam(wil::com_ptr_nothrow<T> value, U** absl_nullable out) {
+  return SaveToOutParam<T, U>(value.detach(), out);
+}
+template <typename T>
+HResult SaveToOutParam(
+    wil::unique_any_t<T> value,
+    typename wil::unique_any_t<T>::pointer* absl_nullable out) {
+  return SaveToOutParam(value.release(), out);
+}
+
+// Saves the value to the out parameter of a COM method. It's a noop if the
+// out parameter is nullptr.
+template <typename T, typename U>
+  requires std::integral<T> && std::integral<U>
+void SaveToOptionalOutParam(T value, U* absl_nullable out);
+
+// Implementations.
+
+template <typename T, typename U>
+HResult SaveToOutParam(T value, U* absl_nullable out) {
+  static_assert(std::convertible_to<T, U>);
+  if (out == nullptr) {
+    return HResultInvalidArg();
+  }
+  *out = value;
+  return HResultOk();
+}
+
+template <typename T, typename U>
+HResult SaveToOutParam(T* absl_nullable value, U** absl_nullable out) {
+  if (out == nullptr) {
+    return HResultInvalidArg();
+  }
+  if (value == nullptr) {
+    return HResultOutOfMemory();
+  }
+  *out = value;
+  return HResultOk();
+}
+
+template <typename T, typename U>
+  requires std::integral<T> && std::integral<U>
+void SaveToOptionalOutParam(T value, U* absl_nullable out) {
+  static_assert(std::convertible_to<T, U>);
+  if (out != nullptr) {
+    *out = value;
+  }
+}
+
 }  // namespace mozc::win32
 
 #endif  // MOZC_BASE_WIN32_COM_H_
