@@ -53,6 +53,7 @@
 #include "base/util.h"
 #include "composer/composer.h"
 #include "composer/table.h"
+#include "converter/candidate.h"
 #include "converter/converter_interface.h"
 #include "converter/segments.h"
 #include "protocol/commands.pb.h"
@@ -62,31 +63,35 @@ namespace mozc {
 namespace quality_regression {
 namespace {
 
-constexpr char kConversionExpect[] = "Conversion Expected";
-constexpr char kConversionNotExpect[] = "Conversion Not Expected";
-constexpr char kConversionMatch[] = "Conversion Match";
-constexpr char kConversionNotMatch[] = "Conversion Not Match";
-constexpr char kReverseConversionExpect[] = "ReverseConversion Expected";
-constexpr char kReverseConversionNotExpect[] = "ReverseConversion Not Expected";
-// For now, suggestion and prediction are using same implementation
-constexpr char kPredictionExpect[] = "Prediction Expected";
-constexpr char kPredictionNotExpect[] = "Prediction Not Expected";
-constexpr char kSuggestionExpect[] = "Suggestion Expected";
-constexpr char kSuggestionNotExpect[] = "Suggestion Not Expected";
-// Zero query
-constexpr char kZeroQueryExpect[] = "ZeroQuery Expected";
-constexpr char kZeroQueryNotExpect[] = "ZeroQuery Not Expected";
+using converter::Candidate;
 
-int GetRank(absl::string_view value, const Segments *segments,
+constexpr absl::string_view kConversionExpect = "Conversion Expected";
+constexpr absl::string_view kConversionNotExpect = "Conversion Not Expected";
+constexpr absl::string_view kConversionMatch = "Conversion Match";
+constexpr absl::string_view kConversionNotMatch = "Conversion Not Match";
+constexpr absl::string_view kReverseConversionExpect =
+    "ReverseConversion Expected";
+constexpr absl::string_view kReverseConversionNotExpect =
+    "ReverseConversion Not Expected";
+// For now, suggestion and prediction are using same implementation
+constexpr absl::string_view kPredictionExpect = "Prediction Expected";
+constexpr absl::string_view kPredictionNotExpect = "Prediction Not Expected";
+constexpr absl::string_view kSuggestionExpect = "Suggestion Expected";
+constexpr absl::string_view kSuggestionNotExpect = "Suggestion Not Expected";
+// Zero query
+constexpr absl::string_view kZeroQueryExpect = "ZeroQuery Expected";
+constexpr absl::string_view kZeroQueryNotExpect = "ZeroQuery Not Expected";
+
+int GetRank(absl::string_view value, const Segments* segments,
             size_t current_segment) {
   if (current_segment == segments->segments_size()) {
     return value.empty() ? 0 : -1;
   }
-  const Segment &seg = segments->segment(current_segment);
+  const Segment& seg = segments->segment(current_segment);
   int rank = 0;
   absl::flat_hash_set<absl::string_view> dedup;
-  for (const Segment::Candidate *cand : seg.candidates()) {
-    const std::string &cand_value = cand->value;
+  for (const Candidate* cand : seg.candidates()) {
+    const std::string& cand_value = cand->value;
     const bool new_value = dedup.insert(cand_value).second;
     if (!new_value) {
       // The value is duplicated and already checked.
@@ -136,7 +141,7 @@ std::string QualityRegressionUtil::TestItem::OutputAsTSV() const {
 }
 
 absl::Status QualityRegressionUtil::TestItem::ParseFromTSV(
-    const std::string &line) {
+    const std::string& line) {
   std::vector<absl::string_view> tokens =
       absl::StrSplit(line, '\t', absl::SkipEmpty());
   if (tokens.size() < 4) {
@@ -190,8 +195,8 @@ QualityRegressionUtil::QualityRegressionUtil(
 
 namespace {
 absl::Status ParseFileInternal(
-    const std::string &filename,
-    std::vector<QualityRegressionUtil::TestItem> *outputs) {
+    const std::string& filename,
+    std::vector<QualityRegressionUtil::TestItem>* outputs) {
   // TODO(taku): support an XML file of Mozcsu.
   InputFileStream ifs(filename);
   if (!ifs.good()) {
@@ -214,17 +219,17 @@ absl::Status ParseFileInternal(
 }  // namespace
 
 // static
-absl::Status QualityRegressionUtil::ParseFile(const std::string &filename,
-                                              std::vector<TestItem> *outputs) {
+absl::Status QualityRegressionUtil::ParseFile(const std::string& filename,
+                                              std::vector<TestItem>* outputs) {
   outputs->clear();
   return ParseFileInternal(filename, outputs);
 }
 
 // static
 absl::Status QualityRegressionUtil::ParseFiles(
-    absl::Span<const std::string> filenames, std::vector<TestItem> *outputs) {
+    absl::Span<const std::string> filenames, std::vector<TestItem>* outputs) {
   outputs->clear();
-  for (const std::string &filename : filenames) {
+  for (const std::string& filename : filenames) {
     const absl::Status result = ParseFileInternal(filename, outputs);
     if (!result.ok()) {
       return result;
@@ -234,10 +239,10 @@ absl::Status QualityRegressionUtil::ParseFiles(
 }
 
 absl::StatusOr<bool> QualityRegressionUtil::ConvertAndTest(
-    const TestItem &item, std::string *actual_value) {
-  const std::string &key = item.key;
-  const std::string &expected_value = item.expected_value;
-  const std::string &command = item.command;
+    const TestItem& item, std::string* actual_value) {
+  const std::string& key = item.key;
+  const std::string& expected_value = item.expected_value;
+  const std::string& command = item.command;
   const int expected_rank = item.expected_rank;
 
   CHECK(actual_value);
@@ -353,8 +358,8 @@ absl::StatusOr<bool> QualityRegressionUtil::ConvertAndTest(
     return true;
   }
 
-  for (const Segment &segment : segments_) {
-    *actual_value += segment.candidate(0).value;
+  for (const Segment& segment : segments_) {
+    absl::StrAppend(actual_value, segment.candidate(0).value);
   }
 
   if (command == kConversionMatch) {
@@ -378,11 +383,11 @@ absl::StatusOr<bool> QualityRegressionUtil::ConvertAndTest(
   return result;
 }
 
-void QualityRegressionUtil::SetRequest(const commands::Request &request) {
+void QualityRegressionUtil::SetRequest(const commands::Request& request) {
   request_ = request;
 }
 
-void QualityRegressionUtil::SetConfig(const config::Config &config) {
+void QualityRegressionUtil::SetConfig(const config::Config& config) {
   config_ = config;
 }
 
