@@ -34,206 +34,44 @@
 #include <cstdint>
 #include <deque>
 #include <iterator>
-#include <limits>
 #include <memory>
+#include <numeric>
 #include <ostream>
 #include <sstream>  // For DebugString()
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "base/logging.h"
-#include "base/number_util.h"
-#include "absl/strings/str_format.h"
-#include "absl/strings/string_view.h"
-
-#ifndef NDEBUG
+#include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/strings/str_cat.h"
-#endif  // !NDEBUG
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
+#include "base/util.h"
+#include "base/vlog.h"
+#include "converter/candidate.h"
 
 namespace mozc {
+namespace converter {
+
 namespace {
 constexpr size_t kMaxHistorySize = 32;
 }  // namespace
 
-void Segment::Candidate::Clear() {
-  key.clear();
-  value.clear();
-  content_value.clear();
-  content_key.clear();
-  consumed_key_size = 0;
-  prefix.clear();
-  suffix.clear();
-  description.clear();
-  usage_title.clear();
-  usage_description.clear();
-  cost = 0;
-  structure_cost = 0;
-  wcost = 0;
-  lid = 0;
-  rid = 0;
-  usage_id = 0;
-  attributes = 0;
-  source_info = SOURCE_INFO_NONE;
-  style = NumberUtil::NumberString::DEFAULT_STYLE;
-  command = DEFAULT_COMMAND;
-  inner_segment_boundary.clear();
-#ifndef NDEBUG
-  log.clear();
-#endif  // NDEBUG
-}
-
-#ifndef NDEBUG
-void Segment::Candidate::Dlog(absl::string_view filename, int line,
-                              absl::string_view message) const {
-  absl::StrAppend(&log, filename, ":", line, " ", message, "\n");
-}
-#endif  // NDEBUG
-
-bool Segment::Candidate::IsValid() const {
-  if (inner_segment_boundary.empty()) {
-    return true;
-  }
-  // The sums of the lengths of key, value components must coincide with those
-  // of key, value, respectively.
-  size_t sum_key_len = 0, sum_value_len = 0;
-  for (InnerSegmentIterator iter(this); !iter.Done(); iter.Next()) {
-    sum_key_len += iter.GetKey().size();
-    sum_value_len += iter.GetValue().size();
-  }
-  return sum_key_len == key.size() && sum_value_len == value.size();
-}
-
-bool Segment::Candidate::EncodeLengths(size_t key_len, size_t value_len,
-                                       size_t content_key_len,
-                                       size_t content_value_len,
-                                       uint32_t *result) {
-  if (key_len > std::numeric_limits<uint8_t>::max() ||
-      value_len > std::numeric_limits<uint8_t>::max() ||
-      content_key_len > std::numeric_limits<uint8_t>::max() ||
-      content_value_len > std::numeric_limits<uint8_t>::max()) {
-    return false;
-  }
-  *result = (static_cast<uint32_t>(key_len) << 24) |
-            (static_cast<uint32_t>(value_len) << 16) |
-            (static_cast<uint32_t>(content_key_len) << 8) |
-            static_cast<uint32_t>(content_value_len);
-  return true;
-}
-
-bool Segment::Candidate::PushBackInnerSegmentBoundary(
-    size_t key_len, size_t value_len, size_t content_key_len,
-    size_t content_value_len) {
-  uint32_t encoded;
-  if (EncodeLengths(key_len, value_len, content_key_len, content_value_len,
-                    &encoded)) {
-    inner_segment_boundary.push_back(encoded);
-    return true;
-  }
-  return false;
-}
-
-std::string Segment::Candidate::DebugString() const {
-  std::stringstream os;
-  os << "(key=" << key << " ckey=" << content_key << " val=" << value
-     << " cval=" << content_value << " cost=" << cost
-     << " scost=" << structure_cost << " wcost=" << wcost << " lid=" << lid
-     << " rid=" << rid << " attributes=" << attributes
-     << " consumed_key_size=" << consumed_key_size;
-  if (!prefix.empty()) {
-    os << " prefix=" << prefix;
-  }
-  if (!suffix.empty()) {
-    os << " suffix=" << suffix;
-  }
-  if (!description.empty()) {
-    os << " description=" << description;
-  }
-  if (!inner_segment_boundary.empty()) {
-    os << " segbdd=";
-    for (size_t i = 0; i < inner_segment_boundary.size(); ++i) {
-      const uint32_t encoded_lengths = inner_segment_boundary[i];
-      const int key_len = encoded_lengths >> 24;
-      const int value_len = (encoded_lengths >> 16) & 0xff;
-      const int content_key_len = (encoded_lengths >> 8) & 0xff;
-      const int content_value_len = encoded_lengths & 0xff;
-      os << absl::StreamFormat("<%d,%d,%d,%d>", key_len, value_len,
-                               content_key_len, content_value_len);
-    }
-  }
-  os << ")" << std::endl;
-  return os.str();
-}
-
-void Segment::Candidate::InnerSegmentIterator::Next() {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_++];
-  key_offset_ += encoded_lengths >> 24;
-  value_offset_ += (encoded_lengths >> 16) & 0xff;
-}
-
-absl::string_view Segment::Candidate::InnerSegmentIterator::GetKey() const {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_];
-  return absl::string_view(key_offset_, encoded_lengths >> 24);
-}
-
-absl::string_view Segment::Candidate::InnerSegmentIterator::GetValue() const {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_];
-  return absl::string_view(value_offset_, (encoded_lengths >> 16) & 0xff);
-}
-
-absl::string_view Segment::Candidate::InnerSegmentIterator::GetContentKey()
-    const {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_];
-  return absl::string_view(key_offset_, (encoded_lengths >> 8) & 0xff);
-}
-
-absl::string_view Segment::Candidate::InnerSegmentIterator::GetContentValue()
-    const {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_];
-  return absl::string_view(value_offset_, encoded_lengths & 0xff);
-}
-
-absl::string_view Segment::Candidate::InnerSegmentIterator::GetFunctionalKey()
-    const {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_];
-  const int key_len = encoded_lengths >> 24;
-  const int content_key_len = (encoded_lengths >> 8) & 0xff;
-  if (const int key_size = key_len - content_key_len; key_size > 0) {
-    return absl::string_view(key_offset_ + content_key_len, key_size);
-  }
-  return absl::string_view();
-}
-
-absl::string_view Segment::Candidate::InnerSegmentIterator::GetFunctionalValue()
-    const {
-  DCHECK_LT(index_, candidate_->inner_segment_boundary.size());
-  const uint32_t encoded_lengths = candidate_->inner_segment_boundary[index_];
-  const int value_len = (encoded_lengths >> 16) & 0xff;
-  const int content_value_len = encoded_lengths & 0xff;
-  if (const int value_size = value_len - content_value_len; value_size > 0) {
-    return absl::string_view(value_offset_ + content_value_len, value_size);
-  }
-  return absl::string_view();
-}
-
-Segment::Segment(const Segment &x)
+Segment::Segment(const Segment& x)
     : removed_candidates_for_debug_(x.removed_candidates_for_debug_),
       segment_type_(x.segment_type_),
       key_(x.key_),
+      key_len_(x.key_len_),
       meta_candidates_(x.meta_candidates_) {
   DeepCopyCandidates(x.candidates_);
 }
 
-Segment &Segment::operator=(const Segment &x) {
+Segment& Segment::operator=(const Segment& x) {
   removed_candidates_for_debug_ = x.removed_candidates_for_debug_;
   segment_type_ = x.segment_type_;
   key_ = x.key_;
+  key_len_ = x.key_len_;
   meta_candidates_ = x.meta_candidates_;
 
   clear_candidates();
@@ -247,23 +85,23 @@ void Segment::clear_candidates() {
   candidates_.clear();
 }
 
-Segment::Candidate *Segment::push_back_candidate() {
+Candidate* Segment::push_back_candidate() {
   auto candidate = std::make_unique<Candidate>();
-  Candidate *ptr = candidate.get();
+  Candidate* ptr = candidate.get();
   pool_.push_back(std::move(candidate));
   candidates_.push_back(ptr);
   return ptr;
 }
 
-Segment::Candidate *Segment::push_front_candidate() {
+Candidate* Segment::push_front_candidate() {
   auto candidate = std::make_unique<Candidate>();
-  Candidate *ptr = candidate.get();
+  Candidate* ptr = candidate.get();
   pool_.push_back(std::move(candidate));
   candidates_.push_front(ptr);
   return ptr;
 }
 
-Segment::Candidate *Segment::insert_candidate(int i) {
+Candidate* Segment::insert_candidate(int i) {
   if (i < 0) {
     LOG(WARNING) << "Invalid insert position [negative]: " << i << " / "
                  << candidates_.size();
@@ -274,14 +112,14 @@ Segment::Candidate *Segment::insert_candidate(int i) {
                 << candidates_.size();
     i = static_cast<int>(candidates_.size());
   }
-  Candidate *candidate =
+  Candidate* candidate =
       pool_.emplace_back(std::make_unique<Candidate>()).get();
   candidates_.insert(candidates_.begin() + i, candidate);
   return candidate;
 }
 
 void Segment::insert_candidate(int i, std::unique_ptr<Candidate> candidate) {
-  Candidate *cand_ptr = pool_.emplace_back(std::move(candidate)).get();
+  Candidate* cand_ptr = pool_.emplace_back(std::move(candidate)).get();
   if (i <= 0) {
     candidates_.push_front(cand_ptr);
   } else if (i >= static_cast<int>(candidates_.size())) {
@@ -303,7 +141,7 @@ void Segment::insert_candidates(
   candidates_.resize(orig_size + candidates.size());
   std::copy_backward(candidates_.begin() + i, candidates_.begin() + orig_size,
                      candidates_.end());
-  for (const auto &candidate : candidates) {
+  for (const auto& candidate : candidates) {
     candidates_[i++] = candidate.get();
   }
 
@@ -343,7 +181,7 @@ void Segment::erase_candidates(int i, size_t size) {
   candidates_.erase(candidates_.begin() + i, candidates_.begin() + end);
 }
 
-const Segment::Candidate &Segment::meta_candidate(size_t i) const {
+const Candidate& Segment::meta_candidate(size_t i) const {
   if (i >= meta_candidates_.size()) {
     LOG(ERROR) << "Invalid index number of meta_candidate: " << i;
     i = 0;
@@ -351,7 +189,7 @@ const Segment::Candidate &Segment::meta_candidate(size_t i) const {
   return meta_candidates_[i];
 }
 
-Segment::Candidate *Segment::mutable_meta_candidate(size_t i) {
+Candidate* Segment::mutable_meta_candidate(size_t i) {
   if (i >= meta_candidates_.size()) {
     LOG(ERROR) << "Invalid index number of meta_candidate: " << i;
     i = 0;
@@ -359,7 +197,7 @@ Segment::Candidate *Segment::mutable_meta_candidate(size_t i) {
   return &meta_candidates_[i];
 }
 
-Segment::Candidate *Segment::add_meta_candidate() {
+Candidate* Segment::add_meta_candidate() {
   return &meta_candidates_.emplace_back();
 }
 
@@ -368,7 +206,7 @@ void Segment::move_candidate(int old_idx, int new_idx) {
   if (old_idx < 0) {
     const int meta_idx = -old_idx - 1;
     DCHECK_LT(meta_idx, meta_candidates_size());
-    Candidate *c = insert_candidate(new_idx);
+    Candidate* c = insert_candidate(new_idx);
     *c = meta_candidates_[meta_idx];
     return;
   }
@@ -376,17 +214,17 @@ void Segment::move_candidate(int old_idx, int new_idx) {
   // normal segment
   if (old_idx < 0 || old_idx >= static_cast<int>(candidates_size()) ||
       new_idx >= static_cast<int>(candidates_size()) || old_idx == new_idx) {
-    VLOG(1) << "old_idx and new_idx are the same";
+    MOZC_VLOG(1) << "old_idx and new_idx are the same";
     return;
   }
   if (old_idx > new_idx) {  // promotion
-    Candidate *c = candidates_[old_idx];
+    Candidate* c = candidates_[old_idx];
     for (int i = old_idx; i >= new_idx + 1; --i) {
       candidates_[i] = candidates_[i - 1];
     }
     candidates_[new_idx] = c;
   } else {  // demotion
-    Candidate *c = candidates_[old_idx];
+    Candidate* c = candidates_[old_idx];
     for (int i = old_idx; i < new_idx; ++i) {
       candidates_[i] = candidates_[i + 1];
     }
@@ -397,14 +235,15 @@ void Segment::move_candidate(int old_idx, int new_idx) {
 void Segment::Clear() {
   clear_candidates();
   key_.clear();
+  key_len_ = 0;
   meta_candidates_.clear();
   segment_type_ = FREE;
 }
 
-void Segment::DeepCopyCandidates(const std::deque<Candidate *> &candidates) {
+void Segment::DeepCopyCandidates(const std::deque<Candidate*>& candidates) {
   DCHECK(pool_.empty());
   pool_.reserve(candidates.size());
-  for (const Candidate *cand : candidates) {
+  for (const Candidate* cand : candidates) {
     auto new_cand = std::make_unique<Candidate>(*cand);
     candidates_.push_back(new_cand.get());
     pool_.push_back(std::move(new_cand));
@@ -428,75 +267,108 @@ std::string Segment::DebugString() const {
   return os.str();
 }
 
-Segments::Segments(const Segments &x)
+Segments::Segments(const Segments& x)
     : max_history_segments_size_(x.max_history_segments_size_),
       resized_(x.resized_),
       pool_(32),
-      revert_entries_(x.revert_entries_),
-      cached_lattice_() {
+      revert_id_(x.revert_id_) {
   // Deep-copy segments.
-  for (const Segment *segment : x.segments_) {
+  for (const Segment* segment : x.segments_) {
     *add_segment() = *segment;
   }
-  // Note: cached_lattice_ is not copied to follow the old copy policy.
-  // TODO(noriyukit): This design is not intuitive. It'd be better to manage
-  // cached_lattice_ in a better way.
 }
 
-Segments &Segments::operator=(const Segments &x) {
+Segments& Segments::operator=(const Segments& x) {
   Clear();
 
   max_history_segments_size_ = x.max_history_segments_size_;
   resized_ = x.resized_;
   // Deep-copy segments.
-  for (const Segment *segment : x.segments_) {
+  for (const Segment* segment : x.segments_) {
     *add_segment() = *segment;
   }
-  revert_entries_ = x.revert_entries_;
-  // Note: cached_lattice_ is not copied; see the comment for the copy
-  // constructor.
+  revert_id_ = x.revert_id_;
   return *this;
 }
 
-Segment *Segments::insert_segment(size_t i) {
-  Segment *segment = pool_.Alloc();
+Segment* Segments::insert_segment(size_t i) {
+  Segment* segment = pool_.Alloc();
   segment->Clear();
   segments_.insert(segments_.begin() + i, segment);
   return segment;
 }
 
-Segment *Segments::push_back_segment() {
-  Segment *segment = pool_.Alloc();
+Segment* Segments::push_back_segment() {
+  Segment* segment = pool_.Alloc();
   segment->Clear();
   segments_.push_back(segment);
   return segment;
 }
 
-Segment *Segments::push_front_segment() {
-  Segment *segment = pool_.Alloc();
+Segment* Segments::push_front_segment() {
+  Segment* segment = pool_.Alloc();
   segment->Clear();
   segments_.push_front(segment);
   return segment;
 }
 
-size_t Segments::history_segments_size() const {
-  size_t result = 0;
-  for (size_t i = 0; i < segments_size(); ++i) {
-    if (segment(i).segment_type() != Segment::HISTORY &&
-        segment(i).segment_type() != Segment::SUBMITTED) {
+// Returns an `Iterator` for the end of history segments.
+template <typename Iterator>
+static Iterator history_segments_end_of(Iterator begin, Iterator end) {
+  Iterator it = begin;
+  for (; it != end; ++it) {
+    const Segment& segment = **it;
+    if (segment.segment_type() != Segment::HISTORY &&
+        segment.segment_type() != Segment::SUBMITTED) {
       break;
     }
-    ++result;
   }
-  return result;
+  return it;
+}
+
+Segments::const_iterator Segments::history_segments_end() const {
+  return const_iterator{
+      history_segments_end_of<decltype(segments_)::const_iterator>(
+          segments_.begin(), segments_.end())};
+}
+
+Segments::iterator Segments::history_segments_end() {
+  return iterator{history_segments_end_of<decltype(segments_)::iterator>(
+      segments_.begin(), segments_.end())};
+}
+
+size_t Segments::history_segments_size() const {
+  const auto end = history_segments_end_of<decltype(segments_)::const_iterator>(
+      segments_.begin(), segments_.end());
+  return end - segments_.begin();
+}
+
+Segments::range Segments::history_segments() {
+  return make_range(begin(), history_segments_end());
+}
+
+Segments::const_range Segments::history_segments() const {
+  return make_range(begin(), history_segments_end());
+}
+
+Segments::range Segments::conversion_segments() {
+  return make_range(history_segments_end(), end());
+}
+
+Segments::const_range Segments::conversion_segments() const {
+  return make_range(history_segments_end(), end());
 }
 
 void Segments::erase_segment(size_t i) {
   if (i >= segments_size()) {
     return;
   }
-  pool_.Release(mutable_segment(i));
-  segments_.erase(segments_.begin() + i);
+  erase_segment(begin() + i);
+}
+
+Segments::iterator Segments::erase_segment(iterator position) {
+  pool_.Release(&*position);
+  return iterator{segments_.erase(position.iterator_)};
 }
 
 void Segments::erase_segments(size_t i, size_t size) {
@@ -504,15 +376,19 @@ void Segments::erase_segments(size_t i, size_t size) {
   if (i >= segments_size() || end > segments_size()) {
     return;
   }
-  for (size_t j = i; j < end; ++j) {
-    pool_.Release(mutable_segment(j));
+  erase_segments(begin() + i, begin() + end);
+}
+
+Segments::iterator Segments::erase_segments(iterator first, iterator last) {
+  for (auto it = first; it != last; ++it) {
+    pool_.Release(&*it);
   }
-  segments_.erase(segments_.begin() + i, segments_.begin() + end);
+  return iterator{segments_.erase(first.iterator_, last.iterator_)};
 }
 
 void Segments::pop_front_segment() {
   if (!segments_.empty()) {
-    Segment *seg = segments_.front();
+    Segment* seg = segments_.front();
     pool_.Release(seg);
     segments_.pop_front();
   }
@@ -520,15 +396,123 @@ void Segments::pop_front_segment() {
 
 void Segments::pop_back_segment() {
   if (!segments_.empty()) {
-    Segment *seg = segments_.back();
+    Segment* seg = segments_.back();
     pool_.Release(seg);
     segments_.pop_back();
   }
 }
 
+void Segments::InitForConvert(absl::string_view key) {
+  set_max_history_segments_size(4);
+  clear_conversion_segments();
+
+  Segment* segment = add_segment();
+  segment->set_key(key);
+  segment->set_segment_type(Segment::FREE);
+
+  MOZC_VLOG(2) << DebugString();
+}
+
+void Segments::InitForCommit(absl::string_view key, absl::string_view value) {
+  clear_conversion_segments();
+
+  Segment* segment = add_segment();
+  segment->set_key(key);
+  segment->set_segment_type(Segment::FIXED_VALUE);
+
+  Candidate* candidate = segment->add_candidate();
+  candidate->key = std::string(key);
+  candidate->content_key = std::string(key);
+  candidate->value = std::string(value);
+  candidate->content_value = std::string(value);
+}
+
 void Segments::Clear() {
   clear_segments();
-  clear_revert_entries();
+  revert_id_ = 0;
+}
+
+void Segments::PrependCandidates(const Segment& previous_segment) {
+  if (conversion_segments_size() == 0) {
+    clear_conversion_segments();
+    Segment* segment = add_segment();
+    segment->set_key(previous_segment.key());
+  }
+
+  DCHECK_EQ(conversion_segments_size(), 1);
+  Segment* segment = mutable_conversion_segment(0);
+  DCHECK(segment);
+
+  const size_t cands_size = previous_segment.candidates_size();
+  for (size_t i = 0; i < cands_size; ++i) {
+    Candidate* candidate = segment->insert_candidate(i);
+    *candidate = previous_segment.candidate(i);
+  }
+  segment->mutable_meta_candidates()->assign(
+      previous_segment.meta_candidates().begin(),
+      previous_segment.meta_candidates().end());
+}
+
+bool Segments::Resize(size_t start_index, absl::Span<const uint8_t> new_sizes) {
+  const size_t total_size =
+      std::accumulate(new_sizes.begin(), new_sizes.end(), 0);
+  if (total_size == 0) {
+    return false;
+  }
+
+  std::string key;
+  size_t key_len = 0;
+  size_t modified_segments_size = 0;
+  for (const Segment& segment : all().drop(start_index)) {
+    absl::StrAppend(&key, segment.key());
+    key_len += segment.key_len();
+    ++modified_segments_size;
+    if (key_len >= total_size) {
+      break;
+    }
+  }
+
+  // If key is empty or less than the total size of new segments, return false.
+  if (key_len == 0 || key_len < total_size) {
+    return false;
+  }
+
+  size_t consumed = 0;
+  std::vector<std::string> new_keys;
+  new_keys.reserve(new_sizes.size());
+
+  for (size_t new_size : new_sizes) {
+    if (new_size != 0 && consumed < key_len) {
+      new_keys.emplace_back(Util::Utf8SubString(key, consumed, new_size));
+      consumed += new_size;
+    }
+  }
+
+  erase_segments(start_index, modified_segments_size);
+
+  for (size_t i = 0; i < new_keys.size(); ++i) {
+    Segment* seg = insert_segment(start_index + i);
+    seg->set_segment_type(Segment::FIXED_BOUNDARY);
+    seg->set_key(std::move(new_keys[i]));
+  }
+
+  // If there is a remaining key, replace the next segment with the new key
+  // prepending the remaining key to the next segment as a FREE type.
+  if (consumed < key_len) {
+    std::string next_segment_key(
+        Util::Utf8SubString(key, consumed, key_len - consumed));
+    const size_t next_segment_index = start_index + new_keys.size();
+    if (next_segment_index < segments_size()) {
+      absl::StrAppend(&next_segment_key, segment(next_segment_index).key());
+      erase_segment(next_segment_index);
+    }
+    Segment* seg = insert_segment(next_segment_index);
+    seg->set_segment_type(Segment::FREE);
+    seg->set_key(next_segment_key);
+  }
+
+  set_resized(true);
+  return true;
 }
 
 void Segments::clear_segments() {
@@ -539,7 +523,7 @@ void Segments::clear_segments() {
 
 void Segments::clear_history_segments() {
   while (!segments_.empty()) {
-    Segment *seg = segments_.front();
+    Segment* seg = segments_.front();
     if (seg->segment_type() != Segment::HISTORY &&
         seg->segment_type() != Segment::SUBMITTED) {
       break;
@@ -549,12 +533,8 @@ void Segments::clear_history_segments() {
 }
 
 void Segments::clear_conversion_segments() {
-  const size_t size = history_segments_size();
-  for (size_t i = size; i < segments_size(); ++i) {
-    pool_.Release(mutable_segment(i));
-  }
   resized_ = false;
-  segments_.resize(size);
+  erase_segments(history_segments_end(), end());
 }
 
 void Segments::set_max_history_segments_size(size_t max_history_segments_size) {
@@ -563,23 +543,14 @@ void Segments::set_max_history_segments_size(size_t max_history_segments_size) {
                std::min(max_history_segments_size, kMaxHistorySize));
 }
 
-Segments::RevertEntry *Segments::push_back_revert_entry() {
-  revert_entries_.resize(revert_entries_.size() + 1);
-  Segments::RevertEntry *entry = &revert_entries_.back();
-  entry->revert_entry_type = Segments::RevertEntry::CREATE_ENTRY;
-  entry->id = 0;
-  entry->timestamp = 0;
-  entry->key.clear();
-  return entry;
-}
-
 std::string Segments::history_key(int size) const {
-  const int hsize = history_segments_size();
-  const int start = size == -1 ? 0 : std::max<int>(0, hsize - size);
+  const_range segments = history_segments();
+  if (size != -1) {
+    segments = segments.take_last(size);
+  }
 
   std::string history_key;
-  for (size_t i = start; i < hsize; ++i) {
-    const Segment &seg = history_segment(i);
+  for (const Segment& seg : segments) {
     if (seg.candidates_size() == 0) continue;
     history_key.append(seg.candidate(0).key);
   }
@@ -588,12 +559,13 @@ std::string Segments::history_key(int size) const {
 }
 
 std::string Segments::history_value(int size) const {
-  const int hsize = history_segments_size();
-  const int start = size == -1 ? 0 : std::max<int>(0, hsize - size);
+  const_range segments = history_segments();
+  if (size != -1) {
+    segments = segments.take_last(size);
+  }
 
   std::string history_value;
-  for (size_t i = start; i < hsize; ++i) {
-    const Segment &seg = history_segment(i);
+  for (const Segment& seg : segments) {
     if (seg.candidates_size() == 0) continue;
     history_value.append(seg.candidate(0).value);
   }
@@ -611,4 +583,5 @@ std::string Segments::DebugString() const {
   return os.str();
 }
 
+}  // namespace converter
 }  // namespace mozc
