@@ -116,15 +116,20 @@ class IPCClientMutexBase {
                      ipc_channel_name, ".ipc");
     std::wstring wmutex_name = Utf8ToWide(mutex_name);
 
-    LPSECURITY_ATTRIBUTES security_attributes_ptr = nullptr;
-    SECURITY_ATTRIBUTES security_attributes;
-    if (!WinSandbox::MakeSecurityAttributes(WinSandbox::kSharableMutex,
-                                            &security_attributes)) {
-      LOG(ERROR) << "Cannot make SecurityAttributes";
-    } else {
-      security_attributes_ptr = &security_attributes;
+    wil::unique_hlocal_security_descriptor security_descriptor =
+        WinSandbox::MakeSecurityDescriptor(WinSandbox::kSharableMutex);
+    if (!security_descriptor) {
+      LOG(ERROR) << "Cannot make SecurityDescriptor";
     }
 
+    SECURITY_ATTRIBUTES security_attributes = {
+        .nLength = sizeof(SECURITY_ATTRIBUTES),
+        .lpSecurityDescriptor = security_descriptor.get(),
+        .bInheritHandle = FALSE,
+    };
+    LPSECURITY_ATTRIBUTES security_attributes_ptr =
+        (security_descriptor ? &security_attributes : nullptr);
+
     // http://msdn.microsoft.com/en-us/library/ms682411(VS.85).aspx:
     // Two or more processes can call CreateMutex to create the same named
     // mutex. The first process actually creates the mutex, and subsequent
@@ -137,9 +142,6 @@ class IPCClientMutexBase {
     // certain which process has initial ownership.
     ipc_mutex_.reset(
         ::CreateMutex(security_attributes_ptr, FALSE, wmutex_name.c_str()));
-    if (security_attributes_ptr != nullptr) {
-      ::LocalFree(security_attributes_ptr->lpSecurityDescriptor);
-    }
 
     const DWORD create_mutex_error = ::GetLastError();
     if (ipc_mutex_.get() == nullptr) {
@@ -461,13 +463,19 @@ IPCServer::IPCServer(const std::string &name, int32_t num_connections,
   }
   DCHECK(!server_address.empty());
 
-  SECURITY_ATTRIBUTES security_attributes;
-  if (!WinSandbox::MakeSecurityAttributes(WinSandbox::kSharablePipe,
-                                          &security_attributes)) {
-    LOG(ERROR) << "Cannot make SecurityAttributes";
+  wil::unique_hlocal_security_descriptor security_descriptor =
+      WinSandbox::MakeSecurityDescriptor(WinSandbox::kSharablePipe);
+  if (!security_descriptor) {
+    LOG(ERROR) << "Cannot make SecurityDescriptor";
     return;
   }
 
+  SECURITY_ATTRIBUTES security_attributes = {
+      .nLength = sizeof(SECURITY_ATTRIBUTES),
+      .lpSecurityDescriptor = security_descriptor.get(),
+      .bInheritHandle = FALSE,
+  };
+
   // Create a named pipe.
   std::wstring wserver_address = Utf8ToWide(server_address);
   HANDLE handle = ::CreateNamedPipe(
@@ -479,7 +487,6 @@ IPCServer::IPCServer(const std::string &name, int32_t num_connections,
       IPC_INITIAL_READ_BUFFER_SIZE, IPC_INITIAL_READ_BUFFER_SIZE, 0,
       &security_attributes);
   const DWORD create_named_pipe_error = ::GetLastError();
-  ::LocalFree(security_attributes.lpSecurityDescriptor);
 
   if (INVALID_HANDLE_VALUE == handle) {
     LOG(FATAL) << "CreateNamedPipe failed" << create_named_pipe_error;
