@@ -33,27 +33,33 @@
 #include <cstdint>
 #include <cstdio>
 #include <optional>
+#include <set>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include "absl/container/flat_hash_map.h"
+#include "absl/container/flat_hash_set.h"
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 #include "base/container/serialized_string_array.h"
 #include "base/japanese_util.h"
-#include "base/logging.h"
 #include "base/number_util.h"
 #include "base/util.h"
+#include "base/vlog.h"
 #include "config/character_form_manager.h"
+#include "converter/attribute.h"
+#include "converter/candidate.h"
 #include "converter/segments.h"
-#include "data_manager/data_manager_interface.h"
+#include "data_manager/data_manager.h"
 #include "dictionary/pos_matcher.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
 #include "request/conversion_request.h"
 #include "rewriter/number_compound_util.h"
 #include "rewriter/rewriter_interface.h"
-#include "absl/container/flat_hash_map.h"
-#include "absl/container/flat_hash_set.h"
-#include "absl/strings/string_view.h"
 
 namespace mozc {
 namespace {
@@ -70,47 +76,56 @@ enum RewriteType {
 
 struct RewriteCandidateInfo {
   RewriteType type;
-  int position;                  // Base (arabic number) candidate position
-  Segment::Candidate candidate;  // Base candidate
+  int position;                    // Base (arabic number) candidate position
+  converter::Candidate candidate;  // Base candidate
 };
 
-bool IsNumberStyleLearningEnabled(const ConversionRequest &request) {
+bool IsNumberStyleLearningEnabled(const ConversionRequest& request) {
   // Enabled in mobile (software keyboard & hardware keyboard)
   return request.request().kana_modifier_insensitive_conversion();
 }
 
-// Returns rewrite type for the given segment and base candidate information.
+// Returns RewriteCandidateInformation if available.
 // base_candidate_pos: the index of the base candidate.
 // *arabic_candidate: arabic candidate using numeric style conversion.
 // POS information, cost, etc will be copied from base candidate.
-RewriteType GetRewriteTypeAndBase(const SerializedStringArray &suffix_array,
-                                  const Segment &seg, int base_candidate_pos,
-                                  const PosMatcher &pos_matcher,
-                                  Segment::Candidate *arabic_candidate) {
-  DCHECK(arabic_candidate);
-
-  const Segment::Candidate &c = seg.candidate(base_candidate_pos);
+std::optional<RewriteCandidateInfo> GetRewriteCandidateInfo(
+    const SerializedStringArray& suffix_array, const Segment& seg,
+    int base_candidate_pos, const PosMatcher& pos_matcher) {
+  const converter::Candidate& c = seg.candidate(base_candidate_pos);
   if (!number_compound_util::IsNumber(suffix_array, pos_matcher, c)) {
-    return NO_REWRITE;
+    return std::nullopt;
+  }
+  if (c.attributes & converter::Attribute::NO_MODIFICATION) {
+    return std::nullopt;
+  }
+  // Do not rewrite hex/oct/bin number.
+  if (c.style == NumberUtil::NumberString::NUMBER_HEX ||
+      c.style == NumberUtil::NumberString::NUMBER_OCT ||
+      c.style == NumberUtil::NumberString::NUMBER_BIN) {
+    return std::nullopt;
   }
 
+  RewriteCandidateInfo info;
+  info.position = base_candidate_pos;
+
   if (Util::GetScriptType(c.content_value) == Util::NUMBER) {
-    *arabic_candidate = c;
-    arabic_candidate->inner_segment_boundary.clear();
-    DCHECK(arabic_candidate->IsValid());
+    info.candidate = c;
+    info.candidate.inner_segment_boundary.clear();
     if (Util::GetScriptType(c.content_key) == Util::NUMBER ||
-        (c.attributes & Segment::Candidate::USER_DICTIONARY)) {
+        (c.attributes & converter::Attribute::USER_DICTIONARY)) {
       // ARABIC_FIRST when:
       // - a user types number key
       // - or, the entry came from the user dictionary
-      return ARABIC_FIRST;
+      info.type = ARABIC_FIRST;
+    } else {
+      info.type = KANJI_FIRST;
     }
-    return KANJI_FIRST;
+    return info;
   }
 
-  std::string half_width_new_content_value;
-  japanese_util::FullWidthToHalfWidth(c.content_key,
-                                      &half_width_new_content_value);
+  const std::string half_width_new_content_value =
+      japanese_util::FullWidthToHalfWidth(c.content_key);
   // Try to get normalized kanji_number and arabic_number.
   // If it failed, do nothing.
   // Retain suffix for later use.
@@ -120,51 +135,60 @@ RewriteType GetRewriteTypeAndBase(const SerializedStringArray &suffix_array,
                                               &kanji_number, &arabic_number,
                                               &number_suffix) ||
       arabic_number == half_width_new_content_value) {
-    return NO_REWRITE;
+    return std::nullopt;
   }
   const std::string new_content_value = arabic_number + number_suffix;
   if (new_content_value == half_width_new_content_value) {
-    return NO_REWRITE;
+    return std::nullopt;
   }
   const std::string suffix(c.value, c.content_value.size(),
                            c.value.size() - c.content_value.size());
-  arabic_candidate->Clear();
-  arabic_candidate->value = new_content_value + suffix;
-  arabic_candidate->content_value = new_content_value;
-  arabic_candidate->key = c.key;
-  arabic_candidate->content_key = c.content_key;
-  arabic_candidate->consumed_key_size = c.consumed_key_size;
-  arabic_candidate->cost = c.cost;
-  arabic_candidate->structure_cost = c.structure_cost;
-  arabic_candidate->lid = c.lid;
-  arabic_candidate->rid = c.rid;
-  arabic_candidate->attributes |=
-      c.attributes & Segment::Candidate::PARTIALLY_KEY_CONSUMED;
-  DCHECK(arabic_candidate->IsValid());
-  return KANJI_FIRST;
+  info.candidate.Clear();
+  info.candidate.value = new_content_value + suffix;
+  info.candidate.content_value = new_content_value;
+  info.candidate.key = c.key;
+  info.candidate.content_key = c.content_key;
+  info.candidate.consumed_key_size = c.consumed_key_size;
+  info.candidate.cost = c.cost;
+  info.candidate.structure_cost = c.structure_cost;
+  info.candidate.lid = c.lid;
+  info.candidate.rid = c.rid;
+  info.candidate.attributes |=
+      c.attributes & converter::Attribute::PARTIALLY_KEY_CONSUMED;
+
+  info.type = KANJI_FIRST;
+  return info;
 }
 
-void GetRewriteCandidateInfos(
-    const SerializedStringArray &suffix_array, const Segment &seg,
-    const PosMatcher &pos_matcher,
-    std::vector<RewriteCandidateInfo> *rewrite_candidate_info) {
-  DCHECK(rewrite_candidate_info);
-  RewriteCandidateInfo info;
+std::vector<RewriteCandidateInfo> GetRewriteCandidateInfos(
+    const SerializedStringArray& suffix_array, const Segment& seg,
+    const PosMatcher& pos_matcher) {
+  std::vector<RewriteCandidateInfo> rewrite_candidate_info;
+  constexpr int kMaxLenForPhoneticNumber = 6;  // "100000" (じゅうまん)
 
   // Use the higher ranked candidate for deciding the insertion position.
   absl::flat_hash_set<std::string> seen;
   for (size_t i = 0; i < seg.candidates_size(); ++i) {
-    const RewriteType type = GetRewriteTypeAndBase(
-        suffix_array, seg, i, pos_matcher, &info.candidate);
-    if (type == NO_REWRITE) {
+    std::optional<RewriteCandidateInfo> info =
+        GetRewriteCandidateInfo(suffix_array, seg, i, pos_matcher);
+    if (!info.has_value()) {
       continue;
     }
-    if (seen.insert(info.candidate.value).second) {
-      info.type = type;
-      info.position = i;
-      rewrite_candidate_info->push_back(info);
+
+    // Skip expanding number variation for large number for phonetic number
+    // candidates. Generating "100000000" for the key "いちおく" would be noisy.
+    const bool is_base_phonetic =
+        (Util::GetFirstScriptType(info->candidate.key) != Util::NUMBER);
+    if (is_base_phonetic &&
+        Util::CharsLen(info->candidate.value) > kMaxLenForPhoneticNumber) {
+      continue;
+    }
+
+    if (seen.insert(info->candidate.value).second) {
+      rewrite_candidate_info.push_back(std::move(*info));
     }
   }
+  return rewrite_candidate_info;
 }
 
 // If top candidate is Kanji numeric, we want to expand at least
@@ -180,9 +204,9 @@ int GetInsertOffset(RewriteType type) {
 void PushBackCandidate(const absl::string_view value,
                        const absl::string_view desc,
                        NumberUtil::NumberString::Style style,
-                       std::vector<Segment::Candidate> *results) {
+                       std::vector<converter::Candidate>* results) {
   bool found = false;
-  for (std::vector<Segment::Candidate>::const_iterator it = results->begin();
+  for (std::vector<converter::Candidate>::const_iterator it = results->begin();
        it != results->end(); ++it) {
     if (it->value == value) {
       found = true;
@@ -190,7 +214,7 @@ void PushBackCandidate(const absl::string_view value,
     }
   }
   if (!found) {
-    Segment::Candidate cand;
+    converter::Candidate cand;
     cand.value = std::string(value);
     cand.description = std::string(desc);
     cand.style = style;
@@ -198,13 +222,13 @@ void PushBackCandidate(const absl::string_view value,
   }
 }
 
-void SetCandidatesInfo(const Segment::Candidate &arabic_cand,
-                       std::vector<Segment::Candidate> *candidates) {
+void SetCandidatesInfo(const converter::Candidate& arabic_cand,
+                       std::vector<converter::Candidate>* candidates) {
   const std::string suffix(
       arabic_cand.value, arabic_cand.content_value.size(),
       arabic_cand.value.size() - arabic_cand.content_value.size());
 
-  for (std::vector<Segment::Candidate>::iterator it = candidates->begin();
+  for (std::vector<converter::Candidate>::iterator it = candidates->begin();
        it != candidates->end(); ++it) {
     it->content_value.assign(it->value);
     it->value.append(suffix);
@@ -214,8 +238,8 @@ void SetCandidatesInfo(const Segment::Candidate &arabic_cand,
 // Note:
 // Some number characters such as superscript (¹) is out of target of
 // number styles.
-bool IsNumberCandidate(const Segment::Candidate &candidate,
-                       const PosMatcher &pos_matcher) {
+bool IsNumberCandidate(const converter::Candidate& candidate,
+                       const PosMatcher& pos_matcher) {
   if (candidate.lid != candidate.rid) {
     return false;
   }
@@ -233,18 +257,18 @@ bool IsNumberCandidate(const Segment::Candidate &candidate,
 }
 
 void SetNumberInfoToExistingCandidates(
-    const std::vector<NumberUtil::NumberString> &numbers,
-    const PosMatcher &pos_matcher, Segment *segment) {
+    absl::Span<const NumberUtil::NumberString> numbers,
+    const PosMatcher& pos_matcher, Segment* segment) {
   absl::flat_hash_map<std::string, NumberUtil::NumberString> number_map;
   // Different number style can have the same surface
   // ex. (123, NUMBER_SEPARATED_ARABIC_HALFWIDTH) and (123, DEFAULT_STYLE)
-  for (const auto &entry : numbers) {
+  for (const auto& entry : numbers) {
     number_map.emplace(entry.value, entry);
   }
 
   for (size_t i = 0; i < segment->candidates_size(); ++i) {
-    Segment::Candidate *candidate = segment->mutable_candidate(i);
-    const auto &itr = number_map.find(candidate->value);
+    converter::Candidate* candidate = segment->mutable_candidate(i);
+    const auto& itr = number_map.find(candidate->value);
     if (itr == number_map.end()) {
       continue;
     }
@@ -262,7 +286,7 @@ void SetNumberInfoToExistingCandidates(
 class CheckValueOperator {
  public:
   explicit CheckValueOperator(const absl::string_view v) : find_value_(v) {}
-  bool operator()(const Segment::Candidate &cand) const {
+  bool operator()(const converter::Candidate& cand) const {
     return (cand.value == find_value_);
   }
 
@@ -272,32 +296,25 @@ class CheckValueOperator {
 
 // If we have the candidates to be inserted before the base candidate,
 // delete them.
-void EraseExistingCandidates(
-    const std::vector<Segment::Candidate> &results, int base_candidate_pos,
-    RewriteType type, Segment *seg,
-    std::vector<RewriteCandidateInfo> *rewrite_candidate_info_list) {
-  DCHECK(seg);
+void FindEraseCandidates(absl::Span<const converter::Candidate> results,
+                         int base_candidate_pos, RewriteType type,
+                         const Segment& seg, std::set<int>* erase_positions) {
   // Remember base candidate value
   const int start_pos = std::min<int>(
-      base_candidate_pos + GetInsertOffset(type), seg->candidates_size() - 1);
+      base_candidate_pos + GetInsertOffset(type), seg.candidates_size() - 1);
   for (int pos = start_pos; pos >= 0; --pos) {
     if (pos == base_candidate_pos) {
       continue;
     }
-    // Simple liner search. |results| size is small. (at most 10 or so)
-    const std::vector<Segment::Candidate>::const_iterator iter =
-        std::find_if(results.begin(), results.end(),
-                     CheckValueOperator(seg->candidate(pos).value));
-    if (iter == results.end()) {
+    if (seg.candidate(pos).attributes & converter::Attribute::NO_MODIFICATION) {
       continue;
     }
 
-    seg->erase_candidate(pos);
-
-    // Adjust position in rewrite_candidate_info.
-    for (size_t i = 0; i < rewrite_candidate_info_list->size(); ++i) {
-      if ((*rewrite_candidate_info_list)[i].position > pos) {
-        --(*rewrite_candidate_info_list)[i].position;
+    // Simple liner search. |results| size is small. (at most 10 or so)
+    for (const converter::Candidate& cand : results) {
+      if (cand.value == seg.candidate(pos).value) {
+        erase_positions->insert(pos);
+        break;
       }
     }
   }
@@ -305,9 +322,9 @@ void EraseExistingCandidates(
 
 // This is a utility function for InsertCandidate and UpdateCandidate.
 // Do not use this function directly.
-void MergeCandidateInfoInternal(const Segment::Candidate &base_cand,
-                                const Segment::Candidate &result_cand,
-                                Segment::Candidate *cand) {
+void MergeCandidateInfoInternal(const converter::Candidate& base_cand,
+                                const converter::Candidate& result_cand,
+                                converter::Candidate* cand) {
   DCHECK(cand);
   cand->key = base_cand.key;
   cand->value = result_cand.value;
@@ -324,44 +341,44 @@ void MergeCandidateInfoInternal(const Segment::Candidate &base_cand,
   if (cand->style == NumberUtil::NumberString::NUMBER_HEX ||
       cand->style == NumberUtil::NumberString::NUMBER_OCT ||
       cand->style == NumberUtil::NumberString::NUMBER_BIN) {
-    cand->attributes |= Segment::Candidate::NO_VARIANTS_EXPANSION;
+    cand->attributes |= converter::Attribute::NO_VARIANTS_EXPANSION;
   }
   cand->attributes |=
-      base_cand.attributes & (Segment::Candidate::PARTIALLY_KEY_CONSUMED |
-                              Segment::Candidate::NO_LEARNING);
+      base_cand.attributes & (converter::Attribute::PARTIALLY_KEY_CONSUMED |
+                              converter::Attribute::NO_LEARNING);
   cand->attributes |=
-      result_cand.attributes & Segment::Candidate::NO_VARIANTS_EXPANSION;
+      result_cand.attributes & converter::Attribute::NO_VARIANTS_EXPANSION;
 }
 
-void InsertCandidate(Segment *segment, int32_t insert_position,
-                     const Segment::Candidate &base_cand,
-                     const Segment::Candidate &result_cand) {
+void InsertCandidate(Segment* segment, int32_t insert_position,
+                     const converter::Candidate& base_cand,
+                     const converter::Candidate& result_cand) {
   DCHECK(segment);
-  Segment::Candidate *c = segment->insert_candidate(insert_position);
+  converter::Candidate* c = segment->insert_candidate(insert_position);
   MergeCandidateInfoInternal(base_cand, result_cand, c);
 }
 
-void UpdateCandidate(Segment *segment, int32_t update_position,
-                     const Segment::Candidate &base_cand,
-                     const Segment::Candidate &result_cand) {
+void UpdateCandidate(Segment* segment, int32_t update_position,
+                     const converter::Candidate& base_cand,
+                     const converter::Candidate& result_cand) {
   DCHECK(segment);
-  Segment::Candidate *c = segment->mutable_candidate(update_position);
+  converter::Candidate* c = segment->mutable_candidate(update_position);
   // Do not call |c->Init()| for an existing candidate.
   // There are two major reasons.
   // 1) Future design change may introduce another field into
-  //    Segment::Candidate. In such situation, simply calling |c->Init()|
+  //    converter::Candidate. In such situation, simply calling |c->Init()|
   //    for an existing candidate may result in unexpeced data loss.
   // 2) In order to preserve existing attribute information such as
-  //    Segment::Candidate::USER_DICTIONARY bit in |c|, we cannot call
+  //    converter::Attribute::USER_DICTIONARY bit in |c|, we cannot call
   //    |c->Init()|. Note that neither |base_cand| nor |result[0]| has
   //    valid value in its |attributes|.
   MergeCandidateInfoInternal(base_cand, result_cand, c);
 }
 
-void InsertConvertedCandidates(const std::vector<Segment::Candidate> &results,
-                               const Segment::Candidate &base_cand,
+void InsertConvertedCandidates(absl::Span<const converter::Candidate> results,
+                               const converter::Candidate& base_cand,
                                int base_candidate_pos, int insert_pos,
-                               Segment *seg) {
+                               Segment* seg) {
   if (results.empty()) {
     return;
   }
@@ -379,8 +396,8 @@ void InsertConvertedCandidates(const std::vector<Segment::Candidate> &results,
   {
     const absl::string_view base_value =
         seg->candidate(base_candidate_pos).value;
-    std::vector<Segment::Candidate>::const_iterator itr = std::find_if(
-        results.begin(), results.end(), CheckValueOperator(base_value));
+    const auto itr = std::find_if(results.begin(), results.end(),
+                                  CheckValueOperator(base_value));
     if (itr != results.end() &&
         itr->style != NumberUtil::NumberString::NUMBER_KANJI &&
         itr->style != NumberUtil::NumberString::NUMBER_KANJI_ARABIC) {
@@ -399,13 +416,13 @@ void InsertConvertedCandidates(const std::vector<Segment::Candidate> &results,
   }
 }
 
-int GetInsertPos(int base_pos, const Segment &segment, RewriteType type) {
+int GetInsertPos(int base_pos, const Segment& segment, RewriteType type) {
   return std::min<int>(base_pos + GetInsertOffset(type),
                        segment.candidates_size());
 }
 
 void InsertHalfArabic(const absl::string_view half_arabic,
-                      std::vector<NumberUtil::NumberString> *output) {
+                      std::vector<NumberUtil::NumberString>* output) {
   output->emplace_back(std::string(half_arabic), "",
                        NumberUtil::NumberString::DEFAULT_STYLE);
 }
@@ -436,12 +453,9 @@ std::vector<NumberUtil::NumberString> GetNumbersInDefaultOrder(
 
 }  // namespace
 
-NumberRewriter::NumberRewriter(const DataManagerInterface *data_manager)
-    : pos_matcher_(data_manager->GetPosMatcherData()) {
-  const char *array = nullptr;
-  size_t size = 0;
-  data_manager->GetCounterSuffixSortedArray(&array, &size);
-  const absl::string_view data(array, size);
+NumberRewriter::NumberRewriter(const DataManager& data_manager)
+    : pos_matcher_(data_manager.GetPosMatcherData()) {
+  absl::string_view data = data_manager.GetCounterSuffixSortedArray();
   // Data manager is responsible for providing a valid data.  Just verify data
   // in debug build.
   DCHECK(SerializedStringArray::VerifyData(data));
@@ -452,57 +466,71 @@ NumberRewriter::NumberRewriter(const DataManagerInterface *data_manager)
 
 NumberRewriter::~NumberRewriter() = default;
 
-int NumberRewriter::capability(const ConversionRequest &request) const {
+int NumberRewriter::capability(const ConversionRequest& request) const {
   if (request.request().mixed_conversion()) {
     return RewriterInterface::ALL;
   }
   return RewriterInterface::CONVERSION;
 }
 
-bool NumberRewriter::Rewrite(const ConversionRequest &request,
-                             Segments *segments) const {
+bool NumberRewriter::Rewrite(const ConversionRequest& request,
+                             Segments* segments) const {
   DCHECK(segments);
   if (!request.config().use_number_conversion()) {
-    VLOG(2) << "no use_number_conversion";
+    MOZC_VLOG(2) << "no use_number_conversion";
     return false;
   }
 
   bool modified = false;
 
-  for (size_t i = 0; i < segments->conversion_segments_size(); ++i) {
-    modified |= RewriteOneSegment(request, i, segments);
+  for (Segment& segment : segments->conversion_segments()) {
+    modified |= RewriteOneSegment(request, &segment, segments);
   }
 
   return modified;
 }
 
-bool NumberRewriter::RewriteOneSegment(const ConversionRequest &request,
-                                       size_t index, Segments *segments) const {
+namespace {
+bool IsAlreadyUpdated(absl::Span<const converter::Candidate> number_candidates,
+                      const Segment& seg) {
+  absl::flat_hash_set<absl::string_view> values;
+  for (int i = 0; i < seg.candidates_size(); ++i) {
+    values.insert(seg.candidate(i).value);
+  }
+
+  for (const auto& candidate : number_candidates) {
+    if (!values.contains(candidate.value)) {
+      return false;
+    }
+  }
+  return true;
+}
+}  // namespace
+
+bool NumberRewriter::RewriteOneSegment(const ConversionRequest& request,
+                                       Segment* seg, Segments* segments) const {
   DCHECK(segments);
   // Radix conversion is done only for conversion mode.
   // Showing radix candidates is annoying for a user.
   const bool exec_radix_conversion =
       (segments->conversion_segments_size() == 1 &&
        request.request_type() == ConversionRequest::CONVERSION);
-  const bool should_rarank = ShouldRerankCandidates(request, *segments);
+  const bool should_rerank = ShouldRerankCandidates(request, *segments);
 
-  Segment *seg = segments->mutable_conversion_segment(index);
+  const std::vector<RewriteCandidateInfo> infos =
+      GetRewriteCandidateInfos(suffix_array_, *seg, pos_matcher_);
 
   bool modified = false;
-  std::vector<RewriteCandidateInfo> rewrite_candidate_infos;
-  GetRewriteCandidateInfos(suffix_array_, *seg, pos_matcher_,
-                           &rewrite_candidate_infos);
-
-  for (int i = rewrite_candidate_infos.size() - 1; i >= 0; --i) {
-    const RewriteCandidateInfo &info = rewrite_candidate_infos[i];
+  std::set<int> erase_positions;  // Use std::set for deterministic order.
+  for (auto it = infos.rbegin(); it != infos.rend(); ++it) {
+    const RewriteCandidateInfo& info = *it;
     if (info.candidate.content_value.size() > info.candidate.value.size()) {
       LOG(ERROR) << "Invalid content_value/value: ";
       break;
     }
 
-    std::string arabic_content_value;
-    japanese_util::FullWidthToHalfWidth(info.candidate.content_value,
-                                        &arabic_content_value);
+    const std::string arabic_content_value =
+        japanese_util::FullWidthToHalfWidth(info.candidate.content_value);
     if (Util::GetScriptType(arabic_content_value) != Util::NUMBER) {
       if (Util::GetFirstScriptType(arabic_content_value) == Util::NUMBER) {
         // Rewrite for number suffix
@@ -516,34 +544,41 @@ bool NumberRewriter::RewriteOneSegment(const ConversionRequest &request,
                  << arabic_content_value;
       break;
     }
-    const std::vector<NumberUtil::NumberString> &output =
+    const std::vector<NumberUtil::NumberString> output =
         GetNumbersInDefaultOrder(info.type, exec_radix_conversion,
                                  arabic_content_value);
     SetNumberInfoToExistingCandidates(output, pos_matcher_, seg);
 
-    const std::vector<Segment::Candidate> &number_candidates =
-        GenerateCandidatesToInsert(info.candidate, output, should_rarank);
+    const std::vector<converter::Candidate> number_candidates =
+        GenerateCandidatesToInsert(info.candidate, output, should_rerank);
 
-    // Caution!!!: This invocation will update the data inside of the
-    // rewrite_candidate_infos. Thus, |info| also can be updated as well
-    // regardless of whether it's const reference-ness.
-    EraseExistingCandidates(number_candidates, info.position, info.type, seg,
-                            &rewrite_candidate_infos);
-    int insert_pos = GetInsertPos(info.position, *seg, info.type);
+    // If all the candidates are already in the segment, do nothing.
+    if (IsAlreadyUpdated(number_candidates, *seg)) {
+      continue;
+    }
+
+    FindEraseCandidates(number_candidates, info.position, info.type, *seg,
+                        &erase_positions);
+    const int insert_pos = GetInsertPos(info.position, *seg, info.type);
     DCHECK_LT(info.position, insert_pos);
     InsertConvertedCandidates(number_candidates, info.candidate, info.position,
                               insert_pos, seg);
     modified = true;
   }
+
+  for (auto it = erase_positions.rbegin(); it != erase_positions.rend(); ++it) {
+    seg->erase_candidate(*it);
+  }
+
   return modified;
 }
 
-std::vector<Segment::Candidate> NumberRewriter::GenerateCandidatesToInsert(
-    const Segment::Candidate &arabic_candidate,
-    const std::vector<NumberUtil::NumberString> &numbers,
+std::vector<converter::Candidate> NumberRewriter::GenerateCandidatesToInsert(
+    const converter::Candidate& arabic_candidate,
+    absl::Span<const NumberUtil::NumberString> numbers,
     bool should_rerank) const {
-  std::vector<Segment::Candidate> converted_numbers;
-  for (const auto &number_string : numbers) {
+  std::vector<converter::Candidate> converted_numbers;
+  for (const auto& number_string : numbers) {
     PushBackCandidate(number_string.value, number_string.description,
                       number_string.style, &converted_numbers);
   }
@@ -554,35 +589,35 @@ std::vector<Segment::Candidate> NumberRewriter::GenerateCandidatesToInsert(
   return converted_numbers;
 }
 
-bool NumberRewriter::ShouldRerankCandidates(const ConversionRequest &request,
-                                            const Segments &segments) const {
+bool NumberRewriter::ShouldRerankCandidates(const ConversionRequest& request,
+                                            const Segments& segments) const {
   if (!IsNumberStyleLearningEnabled(request)) {
-    VLOG(2) << "number style learning is not enabled.";
+    MOZC_VLOG(2) << "number style learning is not enabled.";
     return false;
   }
-  if (request.config().incognito_mode()) {
-    VLOG(2) << "incognito mode";
+  if (request.incognito_mode()) {
+    MOZC_VLOG(2) << "incognito mode";
     return false;
   }
   if (request.config().history_learning_level() == config::Config::NO_HISTORY) {
-    VLOG(2) << "history learning level is NO_HISTORY";
+    MOZC_VLOG(2) << "history learning level is NO_HISTORY";
     return false;
   }
   if (!request.config().use_history_suggest() &&
       request.request_type() == ConversionRequest::SUGGESTION) {
-    VLOG(2) << "no history suggest";
+    MOZC_VLOG(2) << "no history suggest";
     return false;
   }
   if (segments.conversion_segments_size() != 1) {
     // Rewriting "2|階" to "弐|階" using the history would be noisy.
-    VLOG(2) << "do not apply to the multiple segments.";
+    MOZC_VLOG(2) << "do not apply to the multiple segments.";
     return false;
   }
   return true;
 }
 
 void NumberRewriter::RerankCandidates(
-    std::vector<Segment::Candidate> &candidates) const {
+    std::vector<converter::Candidate>& candidates) const {
   std::optional<const CharacterFormManager::NumberFormStyle> stored_entry =
       CharacterFormManager::GetCharacterFormManager()->GetLastNumberStyle();
   if (!stored_entry.has_value()) {
@@ -607,34 +642,32 @@ void NumberRewriter::RerankCandidates(
 
   // Rerank `top_number_entry` to top.
   std::rotate(candidates.begin(), top_number_entry, top_number_entry + 1);
-  candidates.begin()->attributes |= Segment::Candidate::NO_VARIANTS_EXPANSION;
+  candidates.begin()->attributes |= converter::Attribute::NO_VARIANTS_EXPANSION;
 }
 
-void NumberRewriter::Finish(const ConversionRequest &request,
-                            Segments *segments) {
+void NumberRewriter::Finish(const ConversionRequest& request,
+                            const Segments& segments) {
   if (!IsNumberStyleLearningEnabled(request)) {
-    VLOG(2) << "number style learning is not enabled.";
+    MOZC_VLOG(2) << "number style learning is not enabled.";
     return;
   }
 
-  if (request.config().incognito_mode()) {
-    VLOG(2) << "incognito_mode";
+  if (request.incognito_mode()) {
+    MOZC_VLOG(2) << "incognito_mode";
     return;
   }
 
   if (request.config().history_learning_level() !=
       config::Config::DEFAULT_HISTORY) {
-    VLOG(2) << "history_learning_level is not DEFAULT_HISTORY";
+    MOZC_VLOG(2) << "history_learning_level is not DEFAULT_HISTORY";
     return;
   }
 
-  for (size_t i = segments->history_segments_size();
-       i < segments->segments_size(); ++i) {
-    const Segment &segment = segments->segment(i);
+  for (const Segment& segment : segments.conversion_segments()) {
     if (segment.candidates_size() <= 0 ||
         segment.segment_type() != Segment::FIXED_VALUE ||
         segment.candidate(0).attributes &
-            Segment::Candidate::NO_HISTORY_LEARNING) {
+            converter::Attribute::NO_HISTORY_LEARNING) {
       continue;
     }
     if (segment.candidates_size() == 0) {
@@ -647,7 +680,8 @@ void NumberRewriter::Finish(const ConversionRequest &request,
   }
 }
 
-void NumberRewriter::RememberNumberStyle(const Segment::Candidate &candidate) {
+void NumberRewriter::RememberNumberStyle(
+    const converter::Candidate& candidate) {
   const Util::FormType form = Util::GetFormType(candidate.value);
   CharacterFormManager::NumberFormStyle entry = {
       form == Util::HALF_WIDTH ? config::Config::HALF_WIDTH
